; midiSysex

; SYSEX FORMAT:
;
;	0xF0			- sysex start
;	0x7D			- "experimental" manufactorer's ID
;	0xXX			- "pin code byte 1"
;	0xXX			- "pin code byte 2"
;	0xXX			- "pin code byte 3"
;	0xXX			- "pin code byte 4"
;	Midi Channel	- 
;	Program number	- remark: program number 0 is default and non-reprogrammable
;	Note number		
;	Velocity 1 Timer MSB
;	Velocity 1 Timer LSB	- 14 bit timer resolution
;	Velocity 2 Timer MSB
;	Velocity 2 Timer LSB
;   ..
;	Velocity 127 Timer MSB
;	Velocity 127 Timer LSB
; 	0xF7


		CBLOCK	0X3F8
		SysexInByte1	;	0x7D
		SysexInByte2	;	pin code byte 1
		SysexInByte3	;	pin code byte 2
		SysexInByte4	;	pin code byte 3
		SysexInByte5	;	pin code byte 4		
		SysexInByte6	;	midi channel
		SysexInByte7	;	program number
		SysexInByte8	;	note number
		SysExBuffer:0x100	; 	velo -> timer LUT
		ENDC


MidiInSysExStart:
		MOVLW	0x03;
		MOVWF	SysExPtrH
		MOVLW	0xF8;
		MOVWF	SysExPtrL
	RETURN
	
MidiInSysExByte:
		MOVFF	SysExPtrH, FSR0H
		MOVFF	SysExPtrL, FSR0L
		MOVWF	POSTINC0
		MOVFF	FSR0H, SysExPtrH
		MOVFF	FSR0L, SysExPtrL
		MOVF	FSR0H,W
		SUBLW	0x03
		BZ		MidiInSysExByteRangeOK	
		MOVF	FSR0H,W
		SUBLW	0x04
		BZ		MidiInSysExByteRangeOK	
		MOVF	FSR0H,W
		SUBLW	0x05
		BZ		MidiInSysExByteRangeOK
		BRA		MidiInSysExStart
MidiInSysExByteRangeOK:
	RETURN	

MidiInSysExEnd:
		MOVF	SysExPtrH,W
		SUBLW	0x05
		BNZ		invalidSysEx		; Data length does not match
		MOVF	SysExPtrL,W
		SUBLW	0x00
		BNZ		invalidSysEx		; Data length does not match
		LFSR	FSR0, 0x3F8
		MOVF	POSTINC0,W
		SUBLW	0x7D
		BNZ		invalidSysEx
		MOVF	POSTINC0,W			; pincode byte 1
		SUBLW	SysexPinByte1
		BNZ		invalidSysEx
		MOVF	POSTINC0,W			; pincode byte 2
		SUBLW	SysexPinByte2
		BNZ		invalidSysEx
		MOVF	POSTINC0,W			; pincode byte 3
		SUBLW	SysexPinByte3
		BNZ		invalidSysEx
		MOVF	POSTINC0,W			; pincode byte 4
		SUBLW	SysexPinByte4
		BNZ		invalidSysEx
		MOVF	POSTINC0,W			; MIDI Channel
		SUBLW	MidiChannel
		BNZ		invalidSysEx
		NOP
		MOVF	POSTINC0,W				; Program number
		CALL	ProgramLookup	
		MOVWF	CODE_ADDR_HIGH
		SUBLW	0x10
		BZ		invalidSysEx

		MOVF	POSTINC0,W
		CALL 	NoteLookup
		BZ		invalidSysEx
		DECF	WREG
		ADDWF	CODE_ADDR_HIGH
		
		MOVLW	0x80
		MOVWF	COUNTER
shuffle78:						; restack 7bit MSB-LSB into 8bit LSB-MSB
		BCF		STATUS,C
		RRCF	POSTINC0
		BTFSC	STATUS,C
		BSF		INDF0,7
		MOVF	POSTDEC0,F
				
		MOVFF	POSTINC0,TmpT
		MOVF	POSTDEC0,W
		MOVWF	POSTINC0
		MOVFF	TmpT, POSTINC0
				
		MOVLW	0x04
		MOVWF 	BUFFER_ADDR_HIGH		
		MOVLW	0x00
		MOVWF	CODE_ADDR_UPPER

		DECFSZ	COUNTER
		BRA		shuffle78
									; clear all outputs for safety		
		CLRF	OutsA
		CLRF	OutsB
		CLRF	OutsC
		CLRF	LATA
		CLRF	LATB
		CLRF	LATC

		CALL 	FlashPM256		
									; clear all outputs for safety		
		CLRF	OutsA
		CLRF	OutsB
		CLRF	OutsC
		CLRF	LATA
		CLRF	LATB
		CLRF	LATC
		MOVLW	0x02;
		CALL	LedGlowOut	
	RETURN

invalidSysEx:		
	RETURN


;-------------
; Copy 256 bytes from RAM to Program Flash memory
; Source and target are aligned on a 0x100 block.

FlashPM256:
		MOVLW 	0x00
		MOVWF 	BUFFER_ADDR_LOW
		MOVLW	0x00
		MOVWF	CODE_ADDR_LOW

		CALL	FlashPM64
		MOVLW	0x40
		MOVWF	CODE_ADDR_LOW
		MOVWF	BUFFER_ADDR_LOW
		CALL	FlashPM64
		MOVLW	0x80
		MOVWF	CODE_ADDR_LOW
		MOVWF	BUFFER_ADDR_LOW
		CALL	FlashPM64
		MOVLW	0xC0
		MOVWF	CODE_ADDR_LOW
		MOVWF	BUFFER_ADDR_LOW
		CALL	FlashPM64
	RETURN

FlashPM64:
ERASE_BLOCK
		MOVF 	CODE_ADDR_UPPER,W 	; load TBLPTR with the base
		MOVWF 	TBLPTRU 			; address of the memory block
		MOVF 	CODE_ADDR_HIGH,W
		MOVWF 	TBLPTRH
		MOVF 	CODE_ADDR_LOW,W
		MOVWF 	TBLPTRL
		BSF 	EECON1,EEPGD 		; point to FLASH program memory
		BCF 	EECON1,CFGS 		; access FLASH program memory
		BSF 	EECON1,WREN 		; enable write to memory
		BSF 	EECON1,FREE 		; enable Row Erase operation
		BCF 	INTCON,GIE 			; disable interrupts
		MOVLW 	0x55
		MOVWF 	EECON2 				; write 55h
		MOVLW 	0xAA
		MOVWF 	EECON2 				; write AAh
		BSF 	EECON1,WR 			; start erase (CPU stall)
		BSF 	INTCON,GIE 			; re-enable interrupts
		TBLRD*- 					; dummy read decrement
WRITE_BUFFER_BACK
		MOVLW 	8 					; number of write buffer groups of 8 bytes
		MOVWF 	COUNTER_HI
		MOVF 	BUFFER_ADDR_HIGH,W 	; point to buffer
		MOVWF 	FSR0H
		MOVF 	BUFFER_ADDR_LOW,W
		MOVWF 	FSR0L
PROGRAM_LOOP
		MOVLW 	8 					; number of bytes in holding register
		MOVWF 	COUNTER
WRITE_WORD_TO_HREGS
		MOVF 	POSTINC0, W 		; get low byte of buffer data
		MOVWF 	TABLAT 				; present data to table latch
		TBLWT+* 					; write data, perform a short write
									; to internal TBLWT holding register.
		DECFSZ 	COUNTER 			; loop until buffers are full
		BRA 	WRITE_WORD_TO_HREGS
PROGRAM_MEMORY
		BSF 	EECON1,EEPGD 		; point to FLASH program memory
		BCF 	EECON1,CFGS 		; access FLASH program memory
		BSF 	EECON1,WREN 		; enable write to memory
		BCF 	INTCON,GIE 			; disable interrupts
		MOVLW 	0x55
		MOVWF 	EECON2 				; write 55h
		MOVLW 	0xAA
		MOVWF 	EECON2 				; write AAh
		BSF 	EECON1,WR 			; start program (CPU stall)
		BSF 	INTCON,GIE 			; re-enable interrupts
		DECFSZ 	COUNTER_HI 			; loop until done
		BRA 	PROGRAM_LOOP
		BCF 	EECON1,WREN 		; disable write to memory
; verify
		MOVF 	CODE_ADDR_UPPER,W 	; load TBLPTR with the base
		MOVWF 	TBLPTRU 			; address of the memory block
		MOVF 	CODE_ADDR_HIGH,W
		MOVWF 	TBLPTRH
		MOVF 	CODE_ADDR_LOW,W
		MOVWF 	TBLPTRL
		MOVF 	BUFFER_ADDR_HIGH,W 	; point to buffer
		MOVWF 	FSR0H
		MOVF 	BUFFER_ADDR_LOW,W
		MOVWF 	FSR0L
		MOVLW	0x40
		MOVWF 	COUNTER
verifyLoop:
		TBLRD*+ 
		MOVF	TABLAT,W
		SUBWF	POSTINC0
		BNZ		infiloop
		DECFSZ	COUNTER
		BRA		verifyLoop
	RETURN

infiloop
		MOVLW	0x10;
		CALL	LedGlowIn
		BRA 	infiloop
		
